#include "scbar.h"
#include "handler.h"
#include "key.h"
#include <stdio.h>

cui
cui_scbar_new(cui parent, int x, int y, int w, int h, int max_v, int v)
{
	cui obj = cui_alloc(sizeof(struct cui_scbar));
	cui_scbar_init(obj, parent, x, y, w, h, max_v, v);
	return obj;
}

int
cui_scbar_pg_len(cui obj)
{
	cui_scbar p = (cui_scbar)obj;
	return p->hv == CUI_SCBAR_H ? obj->w : obj->h;
}

int
cui_scbar_thumb_len(cui obj)
{
	cui_scbar p = (cui_scbar)obj;
	int len = cui_scbar_pg_len(obj) - (2 + p->max_v);
	return len < 1 ? 1 : len;
}

int
cui_scbar_thumb_pos_max(cui obj)
{
	return cui_scbar_pg_len(obj) - (2 + cui_scbar_thumb_len(obj));
}

int
cui_scbar_thumb_pos(cui obj)
{
	cui_scbar p = (cui_scbar)obj;
	int pos_max = cui_scbar_thumb_pos_max(obj);
	return p->v * (pos_max + 1) / (p->max_v + 1);
}

int
cui_scbar_thumb_pos_to_val(cui obj, int pos)
{
	cui_scbar p = (cui_scbar)obj;
	int pos_max = cui_scbar_thumb_pos_max(obj);
	int vs = (pos * (p->max_v + 1) + pos_max) / (pos_max + 1);
	int ve = ((pos + 1) * (p->max_v + 1) + pos_max) / (pos_max + 1) - 1;
	return vs + (ve - vs) * pos / pos_max;
}

void
cui_scbar_init(cui obj, cui parent, int x, int y, int w, int h, int max_v, int v)
{
	cui_scbar p = (cui_scbar)obj;
	int pos, len;

	cui_base_init(obj, parent, x, y, w, h);
	p->hv = (w >= h) ? CUI_SCBAR_H : CUI_SCBAR_V;
	p->max_v = max_v;
	p->v = v;
	pos = cui_scbar_thumb_pos(obj);
	len = cui_scbar_thumb_len(obj);
	if(p->hv == CUI_SCBAR_H){
		obj->h = 1;
		p->dec = cui_fillbtn_new(obj, 0, 0, 1, 1, "|");
		p->dec_pg = cui_fillbtn_new(obj, 1, 0, pos, 1, "-");
		p->thumb = cui_fillbtn_new(obj, cui_x2(p->dec_pg), 0, len, 1, "=");
		p->inc_pg = cui_fillbtn_new(obj, cui_x2(p->thumb), 0, obj->w-1 - cui_x2(p->thumb), 1, "-");
		p->inc = cui_fillbtn_new(obj, cui_x2(p->inc_pg), 0, 1, 1, "|");
	}else{
		obj->w = 1;
		p->dec = cui_fillbtn_new(obj, 0, 0, 1, 1, "-");
		p->dec_pg = cui_fillbtn_new(obj, 0, 1, 1, pos, "|");
		p->thumb = cui_fillbtn_new(obj, 0, cui_y2(p->dec_pg), 1, len, "=");
		p->inc_pg = cui_fillbtn_new(obj, 0, cui_y2(p->thumb), 1, obj->h-1 - cui_y2(p->thumb), "|");
		p->inc = cui_fillbtn_new(obj, 0, cui_y2(p->inc_pg), 1, 1, "-");
	}
	p->thumb_drag = FALSE;

	cui_bind(p->dec, CUI_EVT_BUTTON, cui_scbar_btn_hdr, obj);
	cui_bind(p->dec_pg, CUI_EVT_BUTTON, cui_scbar_btn_hdr, obj);
	cui_bind(p->thumb, CUI_EVT_BUTTON, cui_scbar_btn_hdr, obj);
	cui_bind(p->inc_pg, CUI_EVT_BUTTON, cui_scbar_btn_hdr, obj);
	cui_bind(p->inc, CUI_EVT_BUTTON, cui_scbar_btn_hdr, obj);

	cui_bind(p->thumb, CUI_EVT_KEY, cui_scbar_thumb_hdr, obj);

	cui_bind(obj, CUI_EVT_RESIZE, cui_scbar_hdr, NULL);
}

int
cui_scbar_btn_hdr(cui fb, int evt, int val, void *prm)
{
	cui obj = (cui)prm;
	cui_scbar p = (cui_scbar)obj;

	/* evt == CUI_EVT_BUTTON */

	if(fb == p->dec){
		if(p->v > 0){
			p->v--;
			cui_scbar_update(obj);
			cui_handler_call(obj, CUI_EVT_BUTTON, p->v);
		}
	}else if(fb == p->inc){
		if(p->v < p->max_v){
			p->v++;
			cui_scbar_update(obj);
			cui_handler_call(obj, CUI_EVT_BUTTON, p->v);
		}
	}else if(fb == p->dec_pg){
		if(p->v > 0){
			p->v -= cui_scbar_pg_len(obj);
			if(p->v < 0) p->v = 0;
			cui_scbar_update(obj);
			cui_handler_call(obj, CUI_EVT_BUTTON, p->v);
		}
	}else if(fb == p->inc_pg){
		if(p->v < p->max_v){
			p->v += cui_scbar_pg_len(obj);
			if(p->v > p->max_v) p->v = p->max_v;
			cui_scbar_update(obj);
			cui_handler_call(obj, CUI_EVT_BUTTON, p->v);
		}
	}else{ /* fb == p->thumb */
		if(0 <= val && val <= p->max_v){
			p->v = val;
			cui_scbar_update(obj);
			cui_handler_call(obj, CUI_EVT_BUTTON, p->v);
		}
	}
	return TRUE;
}

int
cui_scbar_thumb_hdr(cui fb, int evt, int val, void *prm)
{
	cui obj = (cui)prm;
	cui_scbar p = (cui_scbar)obj;
	cui_fillbtn fillbtn = (cui_fillbtn)fb;
	int pos, pos_max;

	/* evt == CUI_EVT_KEY */

	if(!p->thumb_drag){
		if(val != CUI_KEY_ENTER) return FALSE;
		p->thumb_drag = TRUE;
		p->bak = p->v;
		fillbtn->v = TRUE;
		cui_draw(obj);
		return TRUE;
	}

	/* p->thumb_drag */

	pos_max = cui_scbar_thumb_pos_max(obj);

	switch(val){
	case CUI_KEY_ESC:
		if(p->v != p->bak) cui_handler_call(fb, CUI_EVT_BUTTON, p->bak);
	case CUI_KEY_ENTER:
		p->thumb_drag = FALSE;
		fillbtn->v = FALSE;
		cui_draw(obj);
		break;
	case CUI_KEY_UP:
		if(p->hv == CUI_SCBAR_V){
			pos = fb->y - 1;
			if(pos > 0) cui_handler_call(fb, CUI_EVT_BUTTON, cui_scbar_thumb_pos_to_val(obj, pos-1));
		}
		break;
	case CUI_KEY_DOWN:
		if(p->hv == CUI_SCBAR_V){
			pos = fb->y - 1;
			if(pos < pos_max) cui_handler_call(fb, CUI_EVT_BUTTON, cui_scbar_thumb_pos_to_val(obj, pos+1));
		}
		break;
	case CUI_KEY_LEFT:
		if(p->hv == CUI_SCBAR_H){
			pos = fb->x - 1;
			if(pos > 0) cui_handler_call(fb, CUI_EVT_BUTTON, cui_scbar_thumb_pos_to_val(obj, pos-1));
		}
		break;
	case CUI_KEY_RIGHT:
		if(p->hv == CUI_SCBAR_H){
			pos = fb->x - 1;
			if(pos < pos_max) cui_handler_call(fb, CUI_EVT_BUTTON, cui_scbar_thumb_pos_to_val(obj, pos+1));
		}
		break;
	}	
	return TRUE;
}

void
cui_scbar_update(cui obj)
{
	cui_scbar p = (cui_scbar)obj;
	int pos = cui_scbar_thumb_pos(obj);
	int len = cui_scbar_thumb_len(obj);

	/* evt == CUI_EVT_RESIZE */

	int bak = cui_hide_enter(obj);
	if(p->hv == CUI_SCBAR_H){
		cui_w_set(p->dec_pg, pos);
		cui_x_set(p->thumb, cui_x2(p->dec_pg));
		cui_w_set(p->thumb, len);
		cui_x_set(p->inc_pg, cui_x2(p->thumb));
		cui_w_set(p->inc_pg, obj->w-1 - cui_x2(p->thumb));
		cui_x_set(p->inc, obj->w-1);
	}else{
		cui_h_set(p->dec_pg, pos);
		cui_y_set(p->thumb, cui_y2(p->dec_pg));
		cui_h_set(p->thumb, len);
		cui_y_set(p->inc_pg, cui_y2(p->thumb));
		cui_h_set(p->inc_pg, obj->h-1 - cui_y2(p->thumb));
		cui_y_set(p->inc, obj->h-1);
	}
	cui_hide_exit(obj, bak);
}

int
cui_scbar_hdr(cui obj, int evt, int val, void *prm)
{
	cui_scbar_update(obj);
	return TRUE;
}

int
cui_scbar_val_get(cui obj)
{
	cui_scbar p = (cui_scbar)obj;
	return p->v;
}

void
cui_scbar_val_set(cui obj, int v)
{
	cui_scbar p = (cui_scbar)obj;
	p->v = v;
	cui_scbar_update(obj);
}

int
cui_scbar_max_get(cui obj)
{
	cui_scbar p = (cui_scbar)obj;
	return p->max_v;
}

void
cui_scbar_max_set(cui obj, int max_v)
{
	cui_scbar p = (cui_scbar)obj;
	p->max_v = max_v;
	cui_scbar_update(obj);
}

/* EOF */
